final def out = System.out
final def LS = System.getProperty("line.separator")
final def unique = System.currentTimeMillis()
def workDir = new File('.').canonicalFile

final def stepProps = new Properties();
final def inputPropsFile = new File(args[0]);
final def inputPropsStream = null;
try {
    inputPropsStream = new FileInputStream(inputPropsFile);
    stepProps.load(inputPropsStream);
}
catch (IOException e) {
    throw new RuntimeException(e);
}

final def dirOffset = stepProps['dirOffset']
final def repo      = stepProps['repo']
final def domain    = stepProps['domain']
final def username  = stepProps['username']
final def password  = stepProps['password'] ? stepProps['password'] : stepProps['passwordscript']
final def host      = stepProps['host']
final def port      = stepProps['port']
final def infaHome = stepProps['infaHome'];
final def include = stepProps['include'];
final def exclude = stepProps['exclude'];
final def folderMappingList = stepProps['folderMappingList'];
final def repositoryMappingList = stepProps['repositoryMappingList'];
final def conflictResolutionList = stepProps['conflictResolutionList'];
final def conflictResolutionDefault = stepProps['conflictResolutionDefault'];
final def retainGenSeq = stepProps['retainGenSeq'];

final def inputFile = 'informatica_script.' + unique + '.in'
final def outputFile = 'informatica_script.' + unique + '.out'
final def controlFile = 'informatica_script.' + unique + '.ctl'

final def conflictRuleSet = [
        'Lookup Procedure',
        'Stored Procedure',
        'Expression',
        'Filter',
        'Aggregator',
        'Rank',
        'Normalizer',
        'Router',
        'Sequence',
        'Sorter',
        'update strategy',
        'Custom Transformation',
        'Transaction control',
        'External Procedure',
        'Joiner',
        'Mapping',
        'Mapplet',
        'Source Definition',
        'Target Definition',
        'Session',
        'Workflow',
        'Worklet',
        'Email',
        'SessionConfig'
]

def fileSet = []

if (dirOffset) {
    workDir = new File(workDir, dirOffset).canonicalFile
}

// search for files to process
final def ant = new AntBuilder()
def scanner = ant.fileScanner {
    if (exclude && exclude.trim().length() > 0) {
        fileset(dir: "${workDir.canonicalPath}", includes: "${include.split('\n').join(' ')}", excludes: "${exclude?.split('\n')?.join(' ')}")
    } else {
        fileset(dir: "${workDir.canonicalPath}", includes: "${include.split('\n').join(' ')}")
    }
}

if (!scanner.hasFiles()) {
    println "Did not find any files matching \"${include.split('\n').join(' ')}\" except \"${exclude?.split('\n')?.join(' ')}\" in ${workDir.canonicalPath}!"
}
else {
    for (file in scanner) {
        fileSet << file
    }

    // generate Informatica script
    def script = new File(workDir, inputFile)
    script << "connect -r $repo -n $username -x $password "
    if (domain) {
        script << "-d $domain $LS"
    }
    else {
        script << "-h $host -o $port $LS"
    }

    fileSet.each {
        script << "ObjectImport -i \"$it\" -c ${it.name}.ctl $LS"
    }

    script << "exit"
    println('script content:')
    script.eachLine { line -> println(line) }
    println('')

    // generate control file
    def folderMap = [:] as Map
    def repoMap = [:] as Map
    def conflictMap = [:] as Map
    def foundErrors = false
    def repoFolderSet = [] as Set

    folderMappingList?.split('\n')?.each {
        if (it && it.trim().length() > 0) {
            def index = it.indexOf('=')
            if (index < 0 || index == (it.trim().length() - 1)) {
                foundErrors = true
                println "Found invalid folder mapping: $it"
            }
            else {
                folderMap.put(it.substring(0, index).trim(), it.substring(index + 1).trim())
            }
        }
    }
    repositoryMappingList?.split('\n')?.each {
        if (it && it.trim().length() > 0) {
            def index = it.indexOf('=')
            if (index < 0 || index == (it.trim().length() - 1)) {
                foundErrors = true
                println "Found invalid repository mapping: $it"
            }
            else {
                repoMap.put(it.substring(0, index).trim(), it.substring(index + 1).trim())
            }
        }
    }

    conflictResolutionList?.split('\n')?.each {
        if (it && it.trim().length() > 0) {
            def index = it.indexOf('=')
            if (index < 0 || index == (it.trim().length() - 1)) {
                foundErrors = true
                println "Found invalid Conflict Resolution Rule: $it"
            }
            else {
                conflictMap.put(it.substring(0, index).trim(), it.substring(index + 1).trim())
            }
        }
    }

    if (foundErrors) {
        System.exit 1
    }

    fileSet.each {
        repoFolderSet.clear()
        def control = new File(workDir, it.name + ".ctl")
        control << '<?xml version="1.0" encoding="UTF-8"?>'
        if (retainGenSeq) { 
            control << '<IMPORTPARAMS RETAIN_GENERATED_VALUE="YES">'
        }
        else {
            control << '<IMPORTPARAMS>'
        }

        println "Processing $it"
        def parser = new XmlSlurper(false, false)
        parser.setFeature("http://apache.org/xml/features/nonvalidating/load-external-dtd", false)
        def xml = parser.parse(it)
        xml.'REPOSITORY'.each { repository ->
            def repoName = repository.'@NAME'.text()
            repository.'FOLDER'.each { folder ->
                def folderName = folder.'@NAME'.text()
                if (!repoFolderSet.contains(repoName + folderName)) {
                    repoFolderSet.add(repoName+folderName)
                    control << "<FOLDERMAP SOURCEFOLDERNAME=\"$folderName\" SOURCEREPOSITORYNAME=\"$repoName\" TARGETFOLDERNAME=\"${folderMap.get(folderName)?:folderName}\" TARGETREPOSITORYNAME=\"${repoMap.get(repoName)?:repoName}\"/>"
                }
            }
        }
        control << '<RESOLVECONFLICT>'

        conflictRuleSet.each {
            control << "<TYPEOBJECT OBJECTTYPENAME=\"$it\" RESOLUTION=\"${conflictMap.get(it)?:conflictResolutionDefault}\"/>"
        }

        // add any additional conflict resolutions
        conflictMap.keySet().each {
            if(!conflictRuleSet.contains(it)) {
                control << "<TYPEOBJECT OBJECTTYPENAME=\"$it\" RESOLUTION=\"${conflictMap.get(it)?:conflictResolutionDefault}\"/>"
            }
        }

        control << '</RESOLVECONFLICT>'
        control << '</IMPORTPARAMS>'
    }

    //run the Informatica command
    def command = []
    if (infaHome != null && infaHome != "") {
        command.add(infaHome + File.separator + "server" + File.separator + "bin" + File.separator + "pmrep");
    }
    else {
        command.add('pmrep')
    }
    command.add('run')
    command.add('-o')
    command.add(outputFile)
    command.add('-f')
    command.add(inputFile)
    command.add('-e')
    command.add('-s')

    println('command:')
    println(command.join(' '))
    println('')

    def procBuilder = new ProcessBuilder(command as String[])
    procBuilder.directory(workDir)

    if (infaHome != null && infaHome != "") {
        def env = procBuilder.environment();
        env.put("INFA_HOME", infaHome);

        if (env.get("LD_LIBRARY_PATH") != null && env.get("LD_LIBRARY_PATH") != "") {
            env.put("LD_LIBRARY_PATH", env.get("LD_LIBRARY_PATH") + File.pathSeparator + infaHome + File.separator + "server" + File.separator + "bin");
        }
        else {
            env.put("LD_LIBRARY_PATH", infaHome + File.separator + "server" + File.separator + "bin");
        }

        if (env.get("LIBPATH") != null && env.get("LIBPATH") != "") {
            env.put("LIBPATH", env.get("LIBPATH") + File.pathSeparator + infaHome + File.separator + "server" + File.separator + "bin");
        }
        else {
            env.put("LIBPATH", infaHome + File.separator + "server" + File.separator + "bin");
        }
        println("With extra  Environment : ");
        println("INFA_HOME : " + env.get("INFA_HOME"));
        println("LD_LIBRARY_PATH : " + env.get("LD_LIBRARY_PATH"));
        println("LIBPATH : " + env.get("LIBPATH"));
    }


    def process = procBuilder.start();
    process.consumeProcessOutput(out, out)
    process.getOutputStream().close() // close stdin
    process.waitFor()

    def output = new File(workDir, outputFile)
    Scanner sc = new Scanner(output)
    println('pmrep output:')
    def lastLine = ""
    while (sc.hasNextLine()) {
        lastLine = sc.nextLine()
        println(lastLine)
    }
    println('')
    sc.close()

    script.delete()
    output.delete()


    if (!lastLine || !lastLine.trim().equalsIgnoreCase("exit")) {
        System.exit(1)
    }
    else {
        System.exit(process.exitValue())
    }
}
